"""
Module for Terraform Cloud API Endpoint: Users.
"""

from .endpoint import TFCEndpoint

class TFCUsers(TFCEndpoint):
    """
    `Users API Docs \
        <https://www.terraform.io/docs/cloud/api/users.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._users_api_v2_base_url = f"{self._api_v2_base_url}/users"

    def _required_entitlements(self):
        return []

    def show(self, user_id):
        """
        ``GET /users/:user_id``

        `Users Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/users.html#show-a-user>`_
        """
        url = f"{self._users_api_v2_base_url}/{user_id}"
        return self._show(url)



"""
Module for Terraform Cloud API Endpoint: Config Versions.
"""

from .endpoint import TFCEndpoint

class TFCConfigVersions(TFCEndpoint):
    """
    `Config Versions API Docs \
        <https://www.terraform.io/docs/cloud/api/configuration-versions.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._ws_api_v2_base_url = f"{self._api_v2_base_url}/workspaces"
        self._config_version_api_v2_base_url = f"{self._api_v2_base_url}/configuration-versions"

    def _required_entitlements(self):
        return []

    def list(self, workspace_id, page=None, page_size=None):
        """
        ``GET /workspaces/:workspace_id/configuration-versions``

        `Config Versions List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/configuration-versions.html#list-configuration-versions>`_

        Query Parameter(s) (`details \
            <https://www.terraform.io/docs/cloud/api/configuration-versions.html#query-parameters>`_):
            - ``since`` (Optional)
            - ``page`` (Optional)
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/configuration-versions"
        return self._list(url, page=page, page_size=page_size)

    def show(self, config_version_id):
        """
        ``GET /configuration-versions/:configuration-id``

        `Config Versions Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/configuration-versions.html#show-a-configuration-version>`_
        """
        url = f"{self._config_version_api_v2_base_url}/{config_version_id}"
        return self._show(url)

    def create(self, workspace_id, payload):
        """
        ``POST /workspaces/:workspace_id/configuration-versions``

        `Config Versions Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/configuration-versions.html#create-a-configuration-version>`_

        `Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/configuration-versions.html#sample-payload>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/configuration-versions"
        return self._create(url, payload)

    def upload(self, path_to_tarball, upload_url):
        """
        ``PUT https://archivist.terraform.io/v1/object/<UNIQUE OBJECT ID>``

        `Config Versions Upload API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/configuration-versions.html#upload-configuration-files>`_
        """
        data = None

        with open(path_to_tarball, 'rb') as data_bytes:
            data = data_bytes.read()

        return self._put(upload_url, data=data)



"""
Module for Terraform Cloud API Endpoint: Admin Orgs.
"""

from .endpoint import TFCEndpoint

class TFCAdminOrgs(TFCEndpoint):
    """
    `Admin Orgs API Docs \
        <https://www.terraform.io/docs/cloud/api/admin/organizations.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._endpoint_base_url = f"{self._api_v2_base_url}/admin/organizations"

    def _required_entitlements(self):
        return []

    def destroy(self, org_name):
        """
        ``DELETE /admin/organizations/:name``

        `Admin Destroy Org API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/organizations.html#delete-an-organization>`_
        """
        url = f"{self._endpoint_base_url}/{org_name}"
        return self._destroy(url)

    def list(self):
        """
        ``GET /admin/organizations``

        `Admin List Orgs API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/organizations.html#list-all-organizations>`_

        This endpoint lists all organizations in the Terraform Cloud installation.
        """
        return self._list(self._endpoint_base_url)

    def show(self, org_name):
        """
        ``GET /admin/organizations/:name``

        `Admin Show Org API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/organizations.html#show-an-organization>`_
        """
        url = f"{self._endpoint_base_url}/{org_name}"
        return self._show(url)



"""
Module for Terraform Cloud API Endpoint: Policies.
"""

from .endpoint import TFCEndpoint
from ._constants import Entitlements

class TFCPolicies(TFCEndpoint):
    """
    `Policies API Docs \
        <https://www.terraform.io/docs/cloud/api/policies.html>`_
    """
    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._endpoint_base_url = f"{self._api_v2_base_url}/policies"
        self._org_api_v2_base_url = f"{self._api_v2_base_url}/organizations/{org_name}/policies"

    def _required_entitlements(self):
        return [Entitlements.SENTINEL]

    def create(self, payload):
        """
        ``POST /organizations/:organization_name/policies``

        `Policies Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policies.html#create-a-policy>`_

        `Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policies.html#sample-payload>`_
        """
        return self._create(self._org_api_v2_base_url, payload)

    def list(self, page=None, page_size=None, search=None):
        """
        ``GET /organizations/:organization_name/policies``

        `Policies List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policies.html#list-policies>`_

        Query Parameter(s) (`details \
            <https://www.terraform.io/docs/cloud/api/policies.html#query-parameters>`_):
            - ``page`` (Optional)
            - ``page_size`` (Optional)
            - ``search`` (Optional)
        """
        return self._list(\
            self._org_api_v2_base_url, page=page, page_size=page_size, search=search)

    def show(self, policy_id):
        """
        ``GET /policies/:policy_id``

        `Policies Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policies.html#show-a-policy>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}"
        return self._show(url)

    def update(self, policy_id, payload):
        """
        ``PATCH /policies/:policy_id``

        `Policies Update API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policies.html#update-a-policy>`_

        `Update Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policies.html#sample-payload-2>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}"
        return self._update(url, payload)

    def upload(self, policy_id, payload):
        """
        ``PUT /policies/:policy_id/upload``

        `Policies Upload API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policies.html#upload-a-policy>`_

        `Upload Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policies.html#sample-payload-1>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}/upload"
        return self._put(url, octet=True, data=payload)

    def destroy(self, policy_id):
        """
        ``DELETE /policies/:policy_id``

        `Policies Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policies.html#delete-a-policy>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}"
        return self._destroy(url)



"""
Module for testing the Terraform Cloud API Endpoint: Config Versions.
"""

from .base import TestTFCBaseTestCase


class TestTFCConfigVersions(TestTFCBaseTestCase):
    """
    Class for testing the Terraform Cloud API Endpoint: Config Versions.
    """

    _unittest_name = "cnf-ver"
    _endpoint_being_tested = "config_versions"

    def setUp(self):
        self._ws = self._api.workspaces.create(self._get_ws_without_vcs_create_payload())
        self._ws_id = self._ws["data"]["id"]

    def tearDown(self):
        self._api.workspaces.destroy(workspace_id=self._ws_id)

    def test_config_versions(self):
        """
        Test the Config Versions API endpoints.
        """

        # Create a new config version
        config_version = self._api.config_versions.create(
            self._ws_id, self._get_config_version_create_payload())["data"]
        cv_id = config_version["id"]

        # List all of the config versions for the workspace
        all_config_versions = self._api.config_versions.list(self._ws_id)["data"]

        # Confirm we found the newly created config version
        found_conf_ver = False
        for conf_ver in all_config_versions:
            if cv_id == conf_ver["id"]:
                found_conf_ver = True
                break
        self.assertTrue(found_conf_ver)

        # Confirm the config version status is "pending" or "uploaded" as well
        uploaded_or_pending = \
            all_config_versions[0]["attributes"]["status"] in ["pending", "uploaded"]
        self.assertTrue(uploaded_or_pending)

        # Test the show method on that same config version ID
        shown_config_version = self._api.config_versions.show(cv_id)["data"]

        # Confirm the results match the same ID we looked up
        self.assertEqual(shown_config_version["id"], cv_id)

        # Upload the .tf code and confirm it's been uploaded
        upload_url = shown_config_version["attributes"]["upload-url"]
        self._api.config_versions.upload(
            self._config_version_upload_tarball_path, upload_url)

        config_versions = self._api.config_versions.list(self._ws_id)["data"]
        uploaded_or_pending = \
            config_versions[0]["attributes"]["status"] in ["pending", "uploaded"]
        self.assertTrue(uploaded_or_pending)



"""
Module for Terraform Cloud API Endpoint: Workspaces.
"""

from .endpoint import TFCEndpoint

class TFCWorkspaces(TFCEndpoint):
    """
    `Workspaces API Docs \
        <https://www.terraform.io/docs/cloud/api/workspaces.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._ws_api_v2_base_url = f"{self._api_v2_base_url}/workspaces"
        self._org_api_v2_base_url = f"{self._api_v2_base_url}/organizations/{org_name}/workspaces"

    def _required_entitlements(self):
        return []

    def create(self, payload):
        """
        ``POST /organizations/:organization_name/workspaces``

        `Workspaces Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#create-a-workspace>`_

        `Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#sample-payload>`_
        """
        return self._create(self._org_api_v2_base_url, payload)

    def destroy(self, workspace_id=None, workspace_name=None):
        """
        ``DELETE /organizations/:organization_name/workspaces/:name``
        ``DELETE /workspaces/:workspace_id``

        `Workspaces Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#delete-a-workspace>`_
        """
        if workspace_name is not None:
            url = f"{self._org_api_v2_base_url}/{workspace_name}"
        elif workspace_id is not None:
            url = f"{self._ws_api_v2_base_url}/{workspace_id}"
        else:
            self._logger.error("Arguments workspace_name or workspace_id must be defined")

        return self._destroy(url)

    def force_unlock(self, workspace_id):
        """
        ``POST /workspaces/:workspace_id/actions/force-unlock``

        `Workspaces Force Unlock API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#force-unlock-a-workspace>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/actions/force-unlock"
        return self._post(url)

    def lock(self, workspace_id, payload):
        """
        ``POST /workspaces/:workspace_id/actions/lock``

        `Workspaces Lock API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#lock-a-workspace>`_

        `Lock Sample Payload \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#sample-payload-2>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/actions/lock"
        return self._post(url, data=payload)

    def list(self, page=None, page_size=None):
        """
        ``GET /organizations/:organization_name/workspaces``

        `Workspaces List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#list-workspaces>`_

        Query Parameter(s) (`details \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#query-parameters>`_):
            - ``page`` (Optional)
            - ``page_size`` (Optional)
        """
        return self._list(self._org_api_v2_base_url, page=page, page_size=page_size)

    def show(self, workspace_name=None, workspace_id=None):
        """
        ``GET /organizations/:organization_name/workspaces/:name``
        ``GET /workspaces/:workspace_id``

        `Workspaces Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#show-workspace>`_
        """
        if workspace_name is not None:
            url = f"{self._org_api_v2_base_url}/{workspace_name}"
        elif workspace_id is not None:
            url = f"{self._ws_api_v2_base_url}/{workspace_id}"
        else:
            self._logger.error("Arguments workspace_name or workspace_id must be defined")

        return self._show(url)

    def unlock(self, workspace_id):
        """
        ``POST /workspaces/:workspace_id/actions/unlock``

        `Workspaces Unlock API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#unlock-a-workspace>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/actions/unlock"
        return self._post(url)

    def update(self, payload, workspace_name=None, workspace_id=None):
        """
        ``PATCH /organizations/:organization_name/workspaces/:name``
        ``PATCH /workspaces/:workspace_id``

        `Workspaces Update API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#update-a-workspace>`_

        `Update Sample Payload \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#sample-payload-1>`_
        """
        if workspace_name is not None:
            url = f"{self._org_api_v2_base_url}/{workspace_name}"
        elif workspace_id is not None:
            url = f"{self._ws_api_v2_base_url}/{workspace_id}"
        else:
            self._logger.error("Arguments workspace_name or workspace_id must be defined")
        return self._update(url, payload)


    def assign_ssh_key(self, workspace_id, payload):
        """
        ``PATCH /workspaces/:workspace_id/relationships/ssh-key``

        `Workspaces Assign SSH Key API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#assign-an-ssh-key-to-a-workspace>`_

        `Assign Sample Payload \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#sample-payload-3>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/relationships/ssh-key"
        self._patch(url, data=payload)

    def unassign_ssh_key(self, workspace_id, payload):
        """
        ``PATCH /workspaces/:workspace_id/relationships/ssh-key``

        `Workspaces Unassign SSH Key API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#unassign-an-ssh-key-from-a-workspace>`_

        `Unassign Sample Payload \
            <https://www.terraform.io/docs/cloud/api/workspaces.html#sample-payload-4>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/relationships/ssh-key"
        self._patch(url, data=payload)



"""
Module for testing the Terraform Cloud API Endpoint: Audit Trails.
"""

from .base import TestTFCBaseTestCase


class TestTFCAuditTrails(TestTFCBaseTestCase):
    """
    Class for testing the Terraform Cloud API Endpoint: Audit Trails.
    """

    _unittest_name = "audtrl"
    _endpoint_being_tested = "audit_trails"

    def test_audit_trails(self):
        """
        Test the Audit Trails API endpoints.
        """

        # List the audit trails. This deviates from the standard JSON spec
        # we use across the API, so we'll just assert no response, which
        # means it completed successfully for now.

        audit_trails = self._api.audit_trails.list()["data"]
        self.assertEqual(len(audit_trails), 0)



"""
Module for Terraform Cloud API Endpoint: Audit Trails.
"""

from .endpoint import TFCEndpoint

class TFCAuditTrails(TFCEndpoint):
    """
    `Audit Trails API Docs \
        <https://www.terraform.io/docs/cloud/api/audit-trails.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._audit_trail_api_v2_base_url = f"{self._api_v2_base_url}/organization/audit-trail"

    def _required_entitlements(self):
        return []

    def list(self, since=None, page=None):
        """
        ``GET /organization/audit-trail``

        `Audit Trails List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/audit-trails.html#list-audit-trails>`_

        Query Parameter(s) (`details \
            <https://www.terraform.io/docs/cloud/api/audit-trails.html#query-parameters>`_):
            - ``since`` (Optional)
            - ``page`` (Optional)
        """

        return self._list(self._audit_trail_api_v2_base_url, \
            page=page, since=since)



"""
Module for Terraform Cloud API Endpoint: Policy Sets.
"""

from .endpoint import TFCEndpoint
from ._constants import Entitlements

class TFCPolicySets(TFCEndpoint):
    """
    `Policy Sets API Docs \
        <https://www.terraform.io/docs/cloud/api/policy-sets.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._endpoint_base_url = f"{self._api_v2_base_url}/policy-sets"
        self._pol_set_version_api_v2_base_url = f"{self._api_v2_base_url}/policy-set-versions"
        self._org_api_v2_base_url = f"{self._api_v2_base_url}/organizations/{org_name}/policy-sets"

    def _required_entitlements(self):
        return [Entitlements.SENTINEL]

    def create(self, payload):
        """
        ``POST /organizations/:organization_name/policy-sets``

        `Policy Sets Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#create-a-policy-set>`_

        `Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#sample-payload>`_
        """
        return self._create(self._org_api_v2_base_url, payload)

    def list(self, filters=None, include=None, page=None, page_size=None, search=None):
        """
        ``GET /organizations/:organization_name/policy-sets``

        `Policy Sets List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#list-policy-sets>`_

        Query Parameter(s) (`details \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#query-parameters>`_):
            - ``filter[versioned]`` (Optional)
            - ``include`` (Optional)
            - ``page`` (Optional)
            - ``page_size`` (Optional)
            - ``search`` (Optional)

        Example filter(s):

        .. code-block:: python

            filters = [
                {
                    "keys": ["versioned"],
                    "value": "foo"
                }
            ]
        """
        return self._list(\
            self._org_api_v2_base_url, \
            filters=filters, include=include, \
            page=page, page_size=page_size, search=search)

    def show(self, policy_set_id):
        """
        ``GET /policy-sets/:id``

        `Policy Sets Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#show-a-policy-set>`_
        """
        url = f"{self._endpoint_base_url}/{policy_set_id}"
        return self._show(url)

    def update(self, policy_set_id, payload):
        """
        ``PATCH /policy-sets/:id``

        `Policy Sets Update API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#update-a-policy-set>`_

        `Update Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#sample-payload-1>`_
        """
        url = f"{self._endpoint_base_url}/{policy_set_id}"
        return self._update(url, payload)

    def destroy(self, policy_set_id):
        """
        ``DELETE /policies/:policy_set_id``

        `Policy Sets Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#delete-a-policy-set>`_
        """
        url = f"{self._endpoint_base_url}/{policy_set_id}"
        return self._destroy(url)

    def add_policies_to_set(self, policy_set_id, payload):
        """
        ``POST /policy-sets/:id/relationships/policies``

        `Policy Sets Add Policies to Set API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#add-policies-to-the-policy-set>`_

        `Add Policies to Set Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#sample-payload-2>`_
        """
        url = f"{self._endpoint_base_url}/{policy_set_id}/relationships/policies"
        return self._post(url, data=payload)

    def attach_policy_set_to_workspaces(self, policy_id, payload):
        """
        ``POST /policy-sets/:id/relationships/workspaces``

        `Policy Sets Attach Set to Workspaces API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#attach-a-policy-set-to-workspaces>`_

        `Attach Policy Set to Workspaces Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#sample-payload-3>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}/relationships/workspaces"
        return self._post(url, data=payload)

    def remove_policies_from_set(self, policy_id, payload):
        """
        ``DELETE /policy-sets/:id/relationships/policies``

        `Policy Sets Remove Policies From Set API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#remove-policies-from-the-policy-set>`_

        `Remove Policies from Set Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#sample-payload-4>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}/relationships/policies"
        return self._delete(url, data=payload)

    def detach_policy_set_from_workspaces(self, policy_id, payload):
        """
        ``DELETE /policy-sets/:id/relationships/workspaces``

        `Policy Sets Detach Set from Workspaces API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#detach-the-policy-set-from-workspaces>`_

        `Detach Policy Set From Workspaces Sample Payload \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#sample-payload-5>`_
        """
        url = f"{self._endpoint_base_url}/{policy_id}/relationships/workspaces"
        return self._delete(url, data=payload)

    def create_policy_set_version(self, policy_set_id):
        """
        ``POST /policy-sets/:id/versions``

        `Policy Sets Create Set Version API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#create-a-policy-set-version>`_
        """
        url = f"{self._endpoint_base_url}/{policy_set_id}/versions"
        return self._post(url)

    def show_policy_set_version(self, policy_set_version_id):
        """
        ``GET /policy-set-versions/:id``

        `Policy Sets Show Policy Set Version API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#show-a-policy-set-version>`_
        """
        url = f"{self._pol_set_version_api_v2_base_url}/{policy_set_version_id}"
        return self._get(url)

    def upload(self, path_to_tarball, policy_set_version_id):
        """
        ``PUT {derived_policy_set_upload_url}``
        ``PUT https://archivist.terraform.io/v1/object/<UNIQUE OBJECT ID>``

        `Policy Sets Upload API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/policy-sets.html#upload-policy-set-versions>`_
        """
        url = self.show_policy_set_version(policy_set_version_id)["data"]["links"]["upload"]
        data = None
        with open(path_to_tarball, 'rb') as data_bytes:
            data = data_bytes.read()

        return self._put(url, data=data)



"""
Module for Terraform Cloud API Endpoint: Applies.
"""

from .endpoint import TFCEndpoint
from._constants import Entitlements

class TFCApplies(TFCEndpoint):
    """
    `Applies API Docs \
        <https://www.terraform.io/docs/cloud/api/applies.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._endpoint_base_url = f"{self._api_v2_base_url}/applies"

    def _required_entitlements(self):
        return [Entitlements.OPERATIONS]

    def show(self, apply_id):
        """
        ``GET /applies/:id``

        `Applies Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/applies.html#show-an-apply>`_
        """
        url = f"{self._endpoint_base_url}/{apply_id}"
        return self._show(url)



"""
Module for Terraform Cloud API Endpoint: Org Memberships.
"""

from .endpoint import TFCEndpoint

class TFCOrgMemberships(TFCEndpoint):
    """
    `Org Memberships API Docs \
        <https://www.terraform.io/docs/cloud/api/organization-memberships.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._endpoint_base_url = f"{self._api_v2_base_url}/organization-memberships"
        self._org_base_url = \
            f"{self._api_v2_base_url}/organizations/{org_name}/organization-memberships"

    def _required_entitlements(self):
        return []

    def invite(self, payload):
        """
        ``POST /organizations/:organization_name/organization-memberships``

        `Org Memberships Invite API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#invite-a-user-to-an-organization>`_

        `Invite Sample Payload \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#sample-payload>`_
        """
        return self._create(self._org_base_url, payload)

    def list_for_org(self, query=None, filters=None, page=None, page_size=None):
        """
        ``GET /organizations/:organization_name/organization-memberships``

        `Org Memberships List for Org API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#list-memberships-for-an-organization>`_

        Query Parameter(s) (`details \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#query-parameters>`_):
            - ``query`` (Optional)
            - ``filter[status]`` (Optional)
            - ``page`` (Optional)
            - ``page_size`` (Optional)

        Example filter(s):

        .. code-block:: python

            filters = [
                {
                    "keys": ["status"],
                    "value": "foo"
                }
            ]

        """
        return self._list(\
            self._org_base_url, query=query, filters=filters, page=page, page_size=page_size)

    def list_for_user(self):
        """
        ``GET /organization-memberships``

        `Org Memberships List for User API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#list-user-39-s-own-memberships>`_
        """
        return self._list(self._endpoint_base_url)

    def show(self, org_membership_id):
        """
        ``GET /organization-memberships/:organization_membership_id``

        `Org Memberships Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#show-a-membership>`_
        """
        url = f"{self._endpoint_base_url}/{org_membership_id}"
        return self._show(url)

    def remove(self, org_membership_id):
        """
        ``DELETE /organization-memberships/:organization_membership_id``

        `Org Memberships Remove API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/organization-memberships.html#remove-user-from-organization>`_
        """
        url = f"{self._endpoint_base_url}/{org_membership_id}"
        return self._destroy(url)
